/****************************************************************************************/
/*                                                                     					*/
/*  M16C/62P Group Program Collection                                  					*/
/*                                                                     					*/
/*  FILE NAME : mot.c                           		              					*/
/*  CPU       : This program is the Serial flash boot loader by MOT data transfer   	*/
/*  HISTORY   : 2010.05.15\                                                             */
/*                                                                    	 				*/
/*  Copyright (C) 2006. Renesas Technology Corp.                       					*/
/*  Copyright (C) 2006. Renesas Solutions Corp.                        					*/
/*  All right reserved.                                                					*/
/*                                                                     					*/
/*****************************************************************************************/

/**************************************/
/*    include file                    */
/**************************************/
#include <string.h>
#include "mot.h"
#include "flash_header.h"
#include "serial.h"
#include "command.h"

#pragma SECTION program ram_based_prg

int read_record(char *buf, int len)
{
	union union_c2s c2s;
	char *p;
	char c;

	--len;	/* always leave room for terminating '\0' byte */

	for(p=buf; p<buf+len; ++p) 
	{
		c2s.us = GetByte( 60000 ); //after 60 seconds, timeout
		if(c2s.uc[0] == TIMEOUT)
		    return (-2);
		c = c2s.uc[1];		/* read character		*/
        
		switch (c) 
		{
		    case '\r':
		    case '\n':
			    *p = '\0';
			    return (p - buf);
		    case '\0':
		    case 0x03:			/* ^C - Control C		*/
			    return (-1);
		    default:
			    *p = c;
			    //SendByte(c);
		}

	    /* Check for the console hangup (if any different from serial) */
	    //if (gd->jt[XF_getc] != getc) {
		//if (ctrlc()) {
		//    return (-1);
		//}
	    //}
	}

	/* line too long - truncate */
	*p = '\0';
	return (p - buf);
}

int hex1_bin (char c)
{
	if (c >= '0' && c <= '9')
		return (c - '0');
	if (c >= 'a' && c <= 'f')
		return (c + 10 - 'a');
	if (c >= 'A' && c <= 'F')
		return (c + 10 - 'A');
	return (-1);
}

int hex2_bin (char *s)
{
	int i, j;

	if ((i = hex1_bin(*s++)) < 0) {
		return (-1);
	}
	if ((j = hex1_bin(*s)) < 0) {
		return (-1);
	}

	return ((i<<4) + j);
}

int srec_decode (char *input, int *count, unsigned long *addr, char *data)
{
	int	i;
	int	v;				/* conversion buffer	*/
	int	srec_type;			/* S-Record type	*/
	unsigned char chksum;			/* buffer for checksum	*/

	chksum = 0;

	/* skip anything before 'S', and the 'S' itself.
	 * Return error if not found
	 */

	for (; *input; ++input) {
		if (*input == 'S') {		/* skip 'S' */
			++input;
			break;
		}
	}
	if (*input == '\0') {			/* no more data?	*/
		return (SREC_EMPTY);
	}

	v = *input++;				/* record type		*/

	if ((*count = hex2_bin(input)) < 0) {
		return (SREC_E_NOSREC);
	}

	chksum += *count;
	input  += 2;

	switch (v) {				/* record type		*/

	case '0':				/* start record		*/
		srec_type = SREC_START;		/* 2 byte addr field	*/
		*count   -= 3;			/* - checksum and addr	*/
		break;
	case '1':
		srec_type = SREC_DATA2;		/* 2 byte addr field	*/
		*count   -= 3;			/* - checksum and addr	*/
		break;
	case '2':
		srec_type = SREC_DATA3;		/* 3 byte addr field	*/
		*count   -= 4;			/* - checksum and addr	*/
		break;
	case '3':				/* data record with a	*/
		srec_type = SREC_DATA4;		/* 4 byte addr field	*/
		*count   -= 5;			/* - checksum and addr	*/
		break;
/***	case '4'  ***/
	case '5':			/* count record, addr field contains */
		srec_type = SREC_COUNT;	/* a 2 byte record counter	*/
		*count    = 0;			/* no data		*/
		break;
/***	case '6' -- not used  ***/
	case '7':				/* end record with a	*/
		srec_type = SREC_END4;		/* 4 byte addr field	*/
		*count   -= 5;			/* - checksum and addr	*/
		break;
	case '8':				/* end record with a	*/
		srec_type = SREC_END3;		/* 3 byte addr field	*/
		*count   -= 4;			/* - checksum and addr	*/
		break;
	case '9':				/* end record with a	*/
		srec_type = SREC_END2;		/* 2 byte addr field	*/
		*count   -= 3;			/* - checksum and addr	*/
		break;
	default:
		return (SREC_E_BADTYPE);
	}

	/* read address field */
	*addr = 0;

	switch (v) {
	case '3':				/* 4 byte addr field	*/
	case '7':
		if ((v = hex2_bin(input)) < 0) {
			return (SREC_E_NOSREC);
		}
		*addr  += v;
		chksum += v;
		input  += 2;
		/* FALL THRU */
	case '2':				/* 3 byte addr field	*/
	case '8':
		if ((v = hex2_bin(input)) < 0) {
			return (SREC_E_NOSREC);
		}
		*addr <<= 8;
		*addr  += v;
		chksum += v;
		input  += 2;
		/* FALL THRU */
	case '0':				/* 2 byte addr field	*/
	case '1':
	case '5':
	case '9':
		if ((v = hex2_bin(input)) < 0) {
			return (SREC_E_NOSREC);
		}
		*addr <<= 8;
		*addr  += v;
		chksum += v;
		input  += 2;

		if ((v = hex2_bin(input)) < 0) {
			return (SREC_E_NOSREC);
		}
		*addr <<= 8;
		*addr  += v;
		chksum += v;
		input  += 2;

		break;
	default:
		return (SREC_E_BADTYPE);
	}

	/* convert data and calculate checksum */
	for (i=0; i < *count; ++i) {
		if ((v = hex2_bin(input)) < 0) {
			return (SREC_E_NOSREC);
		}
		data[i] = v;
		chksum += v;
		input  += 2;
	}

	/* read anc check checksum */
	if ((v = hex2_bin(input)) < 0) {
		return (SREC_E_NOSREC);
	}

	if ((unsigned char)v != (unsigned char)~chksum) {
		return (SREC_E_BADCHKS);
	}

	return (srec_type);
}

/***********************************************/
/*    MotDownloadAndProgramFlash function   */
/***********************************************/
unsigned char MotDownloadAndProgramFlash (void)
{
/*
AsciiDownloadAndProgramFlash() takes a memory address as the base address to
which data downloaded is programmed.  The data is downloaded using the XModem
protocol developed in 1982 by Ward Christensen.
The routine detects errors due to timeouts, comms errors or invalid checksums.
The routine reports the following to the caller:
-Success
-Invalid address
-Comms error
-Timeout error
-Failure to program flash


Expects:	
--------
FlashAddress:
32-bit address located in Flash memory space starting on a 32-byte boundary

Returns:
--------
MOT_OK				-	Download and Flash programming performed ok
MOT_ADDRESS_ERROR	-	Address was either not on a 128-bit boundary or not in valid Flash
MOT_COMMS_ERROR		-	Comms parity, framing or overrun error
MOT_TIMEOUT			-	Transmitter did not respond to this receiver
MOT_PROG_FAIL		-	Falied to program one or more bytes of the Flash memory
*/
	union union_c2s c2s;

	signed char i = 0;
	
	int val = 0;
	
	char	record[SREC_MAXRECLEN + 1];	/* buffer for one S-Record	*/
	char	binbuf[SREC_MAXBINLEN];		/* buffer for binary data	*/
	int	type;	
	int	binlen;	
	unsigned long	addr;
	int	line_count =  0;
	
	unsigned char remain_flag = 0;
	unsigned long remain_addr = 0;
	unsigned char remain_data[2];
	
	unsigned char program_status;
	unsigned short delay = 0;
	
	// if required, copy the RAM based program function to RAM
	// in this version the copy procedure has already been done in 'InitCommandHandler' in 'command.c'
			
	// flush the comms rx buffer with a delay of 1 sec
	// function will return when no data has been received for 1 sec
	PurgeComms( 1000 );
		
		
	while(1)
	{
		val = read_record(record, SREC_MAXRECLEN + 1);
		
		SendByte(0x13); //Xoff Pause transmission
		for(delay=0;delay<5000;delay++); //delay 5ms
		if(val==-2)
		{
			SendByte(0x11);//Xon resume transmission
		    return MOT_TIMEOUT;
		}
		
		type = srec_decode(record, &binlen, &addr, binbuf);

		if (type < 0) 
		{
			SendByte(0x11);//Xon resume transmission
			return MOT_COMMS_ERROR;		/* Invalid S-Record		*/
		}

		switch (type) 
		{
		    case SREC_DATA2:
		    case SREC_DATA3:
		    case SREC_DATA4:
			    if(addr%2 == 0)
				{
					if(remain_flag == 1)
					{
						if(addr-remain_addr == 1) //ַ
						{
						}
						else
						{
							remain_data[1] = 0;
							program_status = Program_Bytes(remain_addr, 2, (unsigned short*)remain_data);
							if(PROG_FAIL == program_status)
							{
								SendByte(0x11);//Xon resume transmission
								return MOT_PROG_FAIL;
							}
						}
					}
					else
					{
					}
				}
				else//addr%2 != 0
				{
					if(remain_flag == 1)
					{
						if(addr-remain_addr == 1) //ַ
						{
							addr = remain_addr;
							binlen += 1;
							for(i=binlen-1;i>-1;i--)
							{
								binbuf[i] = binbuf[i-1];
							}
							binbuf[0] = remain_data[0];
						}
						else
						{
							remain_data[1] = 0;
							program_status = Program_Bytes(remain_addr, 2, (unsigned short*)remain_data);
							if(PROG_FAIL == program_status)
							{
								SendByte(0x11);//Xon resume transmission
								return MOT_PROG_FAIL;
							}
							
							addr -= 1;
							binlen += 1;
							for(i=binlen-1;i>-1;i--)
							{
								binbuf[i] = binbuf[i-1];
							}
							binbuf[0] = 0;						
						}
					}
					else
					{
						addr -= 1;
						binlen += 1;
						for(i=binlen-1;i>-1;i--)
						{
							binbuf[i] = binbuf[i-1];
						}
						binbuf[0] = 0;	
					}
				}
					
				if(binlen%2 == 0)
				{
					remain_flag = 0;
				}
				else
				{
					remain_flag = 1;
					remain_addr = addr + binlen -1;
					remain_data[0] = binbuf[binlen -1];
					binlen -= 1;						
				}
				
				program_status = Program_Bytes(addr, binlen, (unsigned short*)binbuf);
				if(PROG_FAIL == program_status)
				{
					SendByte(0x11);//Xon resume transmission
					return MOT_PROG_FAIL;
				}
		        break;
		    case SREC_END2:
		    case SREC_END3:
		    case SREC_END4:
			    if(remain_flag == 1)
				{
					remain_data[1] = 0;
					program_status = Program_Bytes(remain_addr, 2, (unsigned short*)remain_data);
					if(PROG_FAIL == program_status)
					{
						SendByte(0x11);//Xon resume transmission
						return MOT_PROG_FAIL;
					}
					remain_flag = 0;
				}
		        SendByte(0x11);//Xon resume transmission
		        return MOT_OK;
		    case SREC_START:
		        break;
		    default:
		        break;
		}
		
		if (0) 
		{	/* print a '.' every 100 lines */
			if ((++line_count % 50) == 0)
				SendByte('|');
		}
		
		SendByte(0x11);//Xon resume transmission
	}		

	return(MOT_OK);		//	Processing doesn't come here. 
						//	The purpose is to erase the Warning output of the compiler. 
}
